/*
 * sockdata.cpp
 *
 *  Created on: Oct 25, 2012
 *      Author: root
 */

#include "../Includes/qstdinc.h"
#include "../Includes/qbuffer.h"

//////////////////////////////////////////////////////
// Socket data
//////////////////////////////////////////////////////
QBuffer::QBuffer()
{
	m_pData = NULL;
	m_nAllocated = 0;
	m_nSize = 0;
}

QBuffer::QBuffer(uint8_t* pData, int nSize)
{
	m_pData = NULL;
	m_nAllocated = 0;
	m_nSize = 0;

	Add(pData, nSize);
}

QBuffer::~QBuffer()
{
	FreeSpace();
}

void QBuffer::FreeSpace()
{
	if (m_pData != NULL)
	{
		free(m_pData);
	}
	m_pData = NULL;
	m_nAllocated = 0;
	m_nSize = 0;
}

void QBuffer::Add(uint8_t nData)
{
	CheckSpace(1);
	m_pData[m_nSize++] = nData;
}

void QBuffer::Add(uint8_t* pData, int nSize)
{
	CheckSpace(nSize);
	memcpy(&m_pData[m_nSize], pData, nSize);
	m_nSize += nSize;
}

void QBuffer::Add(QByteArray* pData)
{
	Add(pData->data(), (int)pData->size());
}

void QBuffer::Add(QBuffer* pData)
{
	Add(pData->GetData(), pData->GetSize());
}

bool QBuffer::Remove(int nFrom, int nCount, QByteArray* pData)
{
	/* Copy data */
	if (nCount == 0) return true;
	if (pData)
	{
		pData->resize(nCount);
		memcpy(pData->data(), &m_pData[nFrom], nCount);
	}

	/* Move data */
	memmove(&m_pData[nFrom], &m_pData[nFrom + nCount], m_nSize - nFrom - nCount);
	m_nSize -= nCount;
	return true;
}

void QBuffer::Clean()
{
	FreeSpace();
}

void QBuffer::Allocate(int nNewSize)
{
	int nFixedSize = 0;

	/* Zero space */
	if (nNewSize == 0) return;

	/* Calculate new buffer alignment */
	if (nNewSize <= 32768)
	{
		nFixedSize = ((nNewSize + 1023) / 1024) * 1024;
	}
	else if (nNewSize <= 262144)
	{
		nFixedSize = ((nNewSize + 32767) / 32768) * 32768;
	}
	else
	{
		nFixedSize = ((nNewSize + 262143) / 262144) * 262144;
	}

	if (nFixedSize < m_nAllocated) return;

	/* Alloc or realloc */
	uint8_t* pXData = (uint8_t*)malloc(nFixedSize);
	if (m_pData != NULL)
	{
		memcpy(pXData, m_pData, m_nSize);
		free(m_pData);
	}
	m_pData = pXData;
	m_nAllocated = nFixedSize;
}
